function parameters=ModelFitPS2(samplingrate,binning,trend)

% MODELFITPS2 obtaines exponents for model from [1] for Drososophila locomotion data
% by fitting its power spectrum with analytical expression. This code uses 
% FASTLOMB program (http://www.mathworks.com/matlabcentral/fileexchange/22215-lomb-normalized-periodogram)
% that calculates power spectra using Lomb-Scargle method.
% The obtained parameters saved into file "model fit parameters.txt". In
% addition to parameters program will also generate an save 2 figures: 1)
% the power spectrum fit; 2) the data fit.
%
%INPUTS:
%   samplingrate:    data sampling rate in seconds
%   binning:         number of points to be binned together for data
%                    visualization (recommend bin to 20-30 min)
%   trend:           set to 0 if data do not need detrending
%                    set to 1 otherwise
%
%OUTPUTS:
%   parameters:      model parameters obtained from the fitting. Parameters
%                    are otputted in following order: bMD, bMR, bER, bED, T0, 
%                    a1/T0, a2/T0. Parameters bMD-bED are in 1/min, primary 
%                    period (T0) in min, a1/T0 and a2/T0 are unitless. 
%                    (parameters also outputed to file "model fit parameters.txt")
%
%INSTRUCTION
% The program works in folowing steps:
%   1. Select a data file with activity. File should be a single column txt
%   file.
%   2. From the the plotted power spectrum determine the primary period. In
%   order to do that the user should either click with left mouse button on
%   circadian peak, or with right mouse button on peak on second harmonic
%   (around circadian period over 2).
%   3. On the opened data plot user should click on the first morning peak
%   that will be used in the analysis (sometimes it is necessary to skip
%   one or two days). The program will redraw the graph, so that it will start
%   from user picked morning peak. The blue and red lines will show the
%   evening peak and next day morning peak based on determined period.
%   4. On this graph the user will determine the parts of the data for 
%   preliminary fit of the data with model. In order to do that the user
%   should click on following points (in this order):
%       a) Top of the morning peak
%       b) End of the morning peak
%       c) Start of the evening peak
%       d) Top of the evening peak
%       e) End of the evening peak
%       f) Top of the morning peak of the next day
%   5. In the opened window with the power spectrum user will pick points
%   that will be used for fitting the analytical expression for power
%   spectra of the model. To do that user first roughly determin the
%   primary period similar to step 2. The appeared slider will allow to
%   fine tune the primary period value so the fitting points (shown with
%   red circle, will appear after moving the slider) lay closer to tops of 
%   the peaks. I recommend using arrows of slider for more accurate 
%   tunning. 
%   6. After getting the optimal picture click "Accept" button and program 
%   will fit selected points with the analytical expression to determine
%   parameters.
%
%   REFERENCES:
%       1.Lazopulo, A., and Syed, S. (2016). A mathematical model provides 
%         mechanistic links to temporal patterns in Drosophila daily activity. BMC Neurosci. 17, 14.
% 
% A.Lazopulo, Dec 2015

  options=optimoptions(@lsqcurvefit,'MaxFunEvals',5000,'MaxIter',1000);
%% Load data (Instructions part 1)
  [filename,pathname]=uigetfile('.txt', 'Select data files','MultiSelect','off');
  file1=fullfile(pathname,filename);
  [~,name,~]=fileparts(file1);
  X=dlmread(file1);
  X=X(1:end)';
  
%% Find circadian period (Instructions part 2)
% 1) plot Lomb-Scargle periodogram
  t_data=0:(samplingrate):(length(X)-1)*(samplingrate);
  figure;
  [PS,f1,~]=fastlomb(X,t_data,1);
  
  title(['Power spectrum of ',name],'Interpreter', 'none')
  
% 2) click on primary period or second harmonic  
  [T,~,click]=ginput(1);
  if click == 3 % correspond to right click of the mouse
      T=round(T*2)*60;
  else
      T=round(T)*60;
  end
  
%% Data processing 
% 1) bin data  
  if binning~=1
        binner1=binning;
        timeseriestobin1 = X(1:(length(X)-mod(length(X),binner1)));
        binned1 = sum(reshape(timeseriestobin1,binner1,length(timeseriestobin1)/binner1));
        X = binned1;
  end
  X=X';
% 2) detrand data
if trend~=0
  t_binned=0:samplingrate*binning:(length(X)-1)*samplingrate*binning;
  p=polyfit(t_binned',X,2);
  f=polyval(p,t_binned)';
  X=X-f-min(X-f);
end
  
%% Determine start of the data for analysis (Instructions part 3)
%   figure;
  dt=samplingrate*binning/60;
  t=0:dt:dt*(length(X)-1);
  if length(X)>7*T/dt
      plot(t(1:7*T/dt),X(1:7*T/dt))
  else
      plot(t,X)
  end
  title(['activity of ', name],'Interpreter', 'none')
  [start,~]=ginput(1);
  X=X(round(start/dt)+1:end);
  t=0:dt:dt*(length(X)-1);
  
%   figure;
  if length(X)>3*T/dt %Plot maximum 3 days of data for better visualization
      plot(t(1:3*T/dt),X(1:3*T/dt))
      hold on
  else
      plot(t,X)
      hold on
  end
  
  plot([T T],[0 max(X)],'r') % plot red line on the next morning peak
  plot([T/2 T/2],[0 max(X)],'c') %plot cian line on the evening peak
  hold off
  title(['activity of ', name],'Interpreter', 'none')
  
%% Preliminary data fit (Instructions part 4)
% 1) pick points for preliminary data fit
  [points,~]=ginput(6);
  points=round(points);
  for i=1:length(points)
      points(i)=points(i)-mod(points(i),dt);
  end
  a1=((points(2)-points(1)))/T;
  a2=((points(4)-points(3)))/T;
%   a3=round((points(6)-points(5)))/T;

% 2) fit part 1 -> 2
%   t1=t(points(1)/dt+1:points(2)/dt+1)';
  ind1=[];
  for i=1:(length(X)*dt/T)-1
      ind1=[ind1,points(1)/dt+1+(i-1)*T/dt:points(2)/dt+1+(i-1)*T/dt];
  end
  t1=t(ind1)';
  A1=mean(X(points(1)/dt+1:T/dt:end));
  ModelFun1=@(b1,t1)0;
  for i=1:(length(X)*dt/T)
      ModelFun1=@(b1,t1)ModelFun1(b1,t1)+A1*(heaviside(t1-(i-1)*T-points(1))-heaviside(t1-(i-1)*T-points(2))).*(exp(b1*a1*T)-exp(b1*(t1-(i-1)*T)))./(exp(b1*a1*T)-1);
  end
  startingValue1=0.005;
  % b1=nlinfit(t1,X_avg(points(1)/dt+1:points(2)/dt+1),ModelFun1,startingValue1);
  [b1,~]=lsqcurvefit(ModelFun1,startingValue1,t1,X(ind1),[],[],options);
  
% 3) fit part 3 -> 4
%   t3=t(points(3)/dt+1:points(4)/dt+1)';
  ind3=[];
  for i=1:(length(X)*dt/T)-1
      ind3=[ind3,points(3)/dt+1+(i-1)*T/dt:points(4)/dt+1+(i-1)*T/dt];
  end
  ind3=round(ind3);
  t3=t(ind3)';
  A3=mean(X(points(4)/dt+1:T/dt:end));
  ModelFun3=@(b3,t3)0;
  for i=1:(length(X)*dt/T)
      ModelFun3=@(b3,t3)ModelFun3(b3,t3)+A3*(heaviside(t3-(i-1)*T-points(3))-heaviside(t3-(i-1)*T-points(4))).*((1-exp(-(t3-(0.5-a2)*T-(i-1)*T)*b3))./(1-exp(-b3*a2*T)));
  end
  startingValue3=0.005;
%   b3=nlinfit(t3,X_avg(points(3)/dt+1:points(4)/dt+1),ModelFun3,startingValue3);

  [b3,~]=lsqcurvefit(ModelFun3,startingValue3,t3,X(ind3),[],[],options);
  
% 4) fit part 4 -> 5
%   t4=t(points(4)/dt+1:points(5)/dt+1)';
  ind4=[];
  for i=1:(length(X)*dt/T)-1
      ind4=[ind4,points(4)/dt+1+(i-1)*T/dt:points(5)/dt+1+(i-1)*T/dt];
  end
  t4=t(ind4)';
  A4=A3;
%   A4=max(X(ind4));
  ModelFun4=@(b4,t4)0;
  for i=1:(length(X)*dt/T)
      ModelFun4=@(b4,t4)ModelFun4(b4,t4)+A4*(heaviside(t4-(i-1)*T-points(4))-heaviside(t4-(i-1)*T-points(5))).*exp(-b4*(t4-(i-1)*T-points(4)));
  end
  startingValue4=0.01;
%   b4=nlinfit(t4,X_avg(points(4)/dt+1:points(5)/dt+1),ModelFun4,startingValue4);

  [b4,~]=lsqcurvefit(ModelFun4,startingValue4,t4,X(ind4),[],[],options);
  
% 5) fit part 5 -> 6
%   t2=t(points(5)/dt+1:points(6)/dt+1)';
  ind2=[];
  for i=1:(length(X)*dt/T)-1
      ind2=[ind2,points(5)/dt+1+(i-1)*T/dt:points(6)/dt+1+(i-1)*T/dt];
  end
  t2=t(ind2)';
  A2=A1;
  ModelFun2=@(b2,t2)0;
  for i=1:(length(X)*dt/T)
      ModelFun2=@(b2,t2)ModelFun2(b2,t2)+A2*(heaviside(t2-(i-1)*T-points(5))-heaviside(t2-(i-1)*T-points(6))).*(exp(b2*(t2-(i-1)*T-a1*T))-1)/(exp(b2*(1-a1)*T)-1);
  end
  startingValue2=0.005;
%   b2=nlinfit(t2,X_avg(points(5)/dt+1:points(6)/dt+1),ModelFun2,startingValue2);
  [b2,~]=lsqcurvefit(ModelFun2,startingValue2,t2,X(ind2),[],[],options);
  b5=(length(X)*dt/T);
  initial_guess=[b1;b2;b3;b4;b5];
  
%% Plot Lomb-Scargle periodogram and pick fitting points (Instructions part 5)
  graph2=figure;
  hold on
  df=f1(2)-f1(1);
  plot(f1(3:end),PS(3:end));
  title(['Power Spectrum of ', name],'Interpreter', 'none')
  ylabel('Power')
  xlabel('Frequency(Hz)')
  plot([1/(T*60) 1/(T*60)],[0 max(PS(3:end))],'r')
  axis([0,0.0002,0,1.3*max(PS(3:end))])
  hold off
  [freq,~,button]=ginput(1);
  if button ==3
      freq=freq/2;
  else 
      freq=freq;
  end
%% create slider and "Accept button" (Instructions part 5)
  sld = uicontrol('Style', 'slider','Max',1/(12*3600),'Min',1/(36*3600),'SliderStep',[1/96,1/96],'Position', [400 20 120 20],'Value',freq,'Callback', @period); 
  btn = uicontrol('Style', 'pushbutton', 'String', 'Accept','Position', [20 20 50 20],'Callback', @Accept);
 
%% slider function (Instructions part 5)
  function period(source,callbackdata)
        freq=source.Value;
        ind=[];
        for n=1:10
            ind=[ind,find(freq*n-df/2<f1 & f1<freq*n+df/2)];
        end
        ind2=find(f1<0.0002);
        plot(f1(ind2),PS(ind2),f1(ind)',PS(ind),'o')
        linkdata on
        

  end
%% Fitting of power spectra
  function Accept(source,callbackdata)
        freq=get(sld,'Value');
        T=1/(freq*60);
% 1)Power spectrum analitycal expression
        B1=@(b,n)2*b(1)*exp(a1*b(1)*T)*T./((-1+exp(a1*b(1)*T))*(4*n.^2*pi^2+b(1)^2*T^2))+2*b(2)*exp(a1*b(2)*T)*T./((exp(b(2)*T)-exp(a1*b(2)*T))*(4*n.^2*pi^2+b(2)^2*T^2));
        B2=@(b,n)(-1).^n*2*b(3)*exp(a2*b(3)*T)*T./((-1+exp(a2*b(3)*T))*(4*n.^2*pi^2+b(3)^2*T^2));
        B3=@(b,n)T^2*b(1)^2*exp(a1*b(1)*T)./((-1+exp(a1*b(1)*T))*n*pi.*(4*n.^2*pi^2+b(1)^2*T^2))+T^2*b(2)^2*exp(a1*b(2)*T)./((exp(b(2)*T)-exp(a1*b(2)*T))*n*pi.*(4*n.^2*pi^2+b(2)^2*T^2));
        B4=@(b,n)(-1).^n*b(3)^2*T^2*exp(a2*b(3)*T)./((-1+exp(a2*b(3)*T))*n*pi.*(4*n.^2*pi^2+b(3)^2*T^2));
        C1=@(b,n)2*b(1)*T./((-1+exp(a1*b(1)*T))*(4*n.^2*pi^2+b(1)^2*T^2))+2*b(2)*exp(b(2)*T)*T./((exp(b(2)*T)-exp(a1*b(2)*T))*(4*n.^2*pi^2+b(2)^2*T^2))+(-1).^n*2*b(3)*T./((-1+exp(a2*b(3)*T))*(4*n.^2*pi^2+b(3)^2*T^2))+(-1).^n*2*b(4)*T./(4*n.^2*pi^2+b(4)^2*T^2)-2*b(4)*T*exp(-0.5*b(4)*T)./(4*n.^2*pi^2+b(4)^2*T^2);
        C2=@(b,n)exp(a1*b(1)*T)./((-1+exp(a1*b(1)*T))*n*pi)+exp(a1*b(2)*T)./((exp(b(2)*T)-exp(a1*b(2)*T))*n*pi)-4*pi*n./((-1+exp(a1*b(1)*T))*(4*n.^2*pi^2+b(1)^2*T^2))-4*pi*n*exp(b(2)*T)./((exp(b(2)*T)-exp(a1*b(2)*T))*(4*n.^2*pi^2+b(2)^2*T^2))-(-1).^n*exp(a2*b(3)*T)./((-1+exp(a2*b(3)*T))*n*pi)+(-1).^n*4*pi.*n./((-1+exp(a2*b(3)*T))*(4*n.^2*pi^2+b(3)^2*T^2))-4*pi*n*exp(-0.5*b(4)*T)./(4*n.^2*pi^2+b(4)^2*T^2)+(-1).^n*4*pi.*n./(4*n.^2*pi^2+b(4)^2*T^2);
        Real_part=@(b,n)-T*(C1(b,n)-B1(b,n).*cos(2*a1*pi*n)-B2(b,n).*cos(2*a2*pi*n)+B3(b,n).*sin(2*a1*n*pi)+B4(b,n).*sin(2*a2*n*pi))/2;
        Imaginary_part=@(b,n)T*(C2(b,n)-B3(b,n).*cos(2*a1*n*pi)+B4(b,n).*cos(2*a2*n*pi)-B1(b,n).*sin(2*a1*n*pi)+B2(b,n).*sin(2*a2*n*pi))/2;
        ModelPS=@(b,n)b(5)*((Real_part(b,n)).^2+(Imaginary_part(b,n)).^2);
        
% 2)fit of the selected points with analytical expression
        ind=[];
        for n=1:10
            ind=[ind,find(freq*n-df/2<f1 & f1<freq*n+df/2)];
        end
        n=1:length(ind);      
        lb=[];
        ub=[1,1,1,1,+Inf];
        [b,res]=lsqcurvefit(ModelPS,initial_guess,n,PS(ind)',lb,ub,options);
          n=1:60000;
        plot(f1(3:end),PS(3:end),n/(T*60),ModelPS(b,n),'r')
        axis([0,0.0002,0,1.3*max(PS(3:end))])
        dlmwrite('PS_fit.txt',[n'./(T*60),ModelPS(b,n')],'\t');
        %   hold off
        
        
        [b1,b2,b3,b4,b5]=deal(b(1),b(2),b(3),b(4),b(5));
        
%% Generate and plot the model with obtained parameters and its power spectrum%        
        T=round(T);
        t1=0:1:round(a1*T)-1;
        X1=A1*(exp(b1*a1*T)-exp(b1*t1))/(exp(b1*a1*T)-1);
        t2=0:1:T-round(a1*T)-1;
        %X1(N1+1:1440)=0;
        X1(round(a1*T)+1:T)=A2*(exp(b2*t2)-1)/exp(b2*(T-a1*T-1));
        X1=repmat(X1,1,floor(length(X)*dt/T)+1);
        X2=0;
        t3=0:1:round(a2*T)-1;
        X2(1:points(4)-round(a2*T))=0;
        X2(points(4)-round(a2*T)+1:points(4))=A3*(1-exp(-b3*t3))/(1-exp(-b3*(round(a2*T)-1)));
        t4=points(4)+1:1:T-1;
        X2(points(4)+1:T-1)=A4*exp(-b4*(t4-points(4)));
        X2(T)=0;
        X2=repmat(X2,1,floor(length(X)*dt/T)+1);
        
        X_model=(X1+X2);
        dlmwrite('model_fit.txt',X_model');
        t_model=0:1:(length(X_model)-1);
        t_data=0:dt:(length(X)-1)*dt;
        graph1=figure;
        hold on
        plot(t_data,X,'k')
        plot(t_model,X_model,'color','r','LineWidth',2)
        hold off
        figure;
        fastlomb(X_model,t_model*60,1);
        title('model power spectrum','Interpreter', 'none')
%% Save parameters to file
        parameters=[b1,b2,b3,b4,T,a1,a2];
        fid=fopen('model fit parameters.txt','a+');
        fprintf(fid,'%20s \t %1.4f \t %1.4f \t %1.4f \t %1.4f \t %1.4f \t %1.4f \t %1.4f \t %1.4f \r\n', filename, parameters,res/(max(PS))^2);
        fclose(fid);
%% Save the data fit and the power spectrum fit to file
        outputfig1=[pathname,'\',name,'_modelfit2'];
        outputfig2=[pathname,'\',name,'_PSfit2'];
        saveas(graph1,outputfig1,'fig')
        saveas(graph2,outputfig2,'fig')
  end
end
  
  
  
  
  